;Example assembly source for using Math64 functions.
;(86 version)
;The program gets user-input for A and B, then outputs the following
; calculations:
;sqrt(A)
;sqrt(B)
;A^2
;B^7
;A*B
;int A/B
;In doing the above calculations, this program demonstrates how to input and
; displaying variables, as well as square rooting, exponentiating,
; multiplying, and dividing 64-bit variables.
;Note that the exponentiating and multiplying may result in overflows,
; meaning in incorrect calculations; also, since the screen is only 21
; characters wide, the last few digits of those results could be cut off.

#include "ti86asm.inc"

;I'm not sure if this source will compile w/o these equates, but best if
; they're here, I think:
TEXT_MEM	=	_textShadow	;text memory
GET_KEY		=	_getky		;ROM call for getting the last key press
TX_CHARPUT	=	_putc		;ROM call for putting a character to the screen
CLEARLCD	=	_clrLCD		;ROM call for clearing the screen
D_ZT_STR	=	_puts		;ROM call for displaying a ZT string
CURSOR_ROW 	=	_curRow		;RAM address for cursor's row
CURSOR_COL	=	_curCol		;RAM address for cursor's column
GRAPH_MEM	=	_plotSScreen	;graphics memory

;file that has basic function macros
#include "macro64.h"
;file that has complex function macros
#include "cmplx64.h"

;Sets up memory locations of various variables, each 8 bytes long
;This allows for up to about 17 variables in the text memory alone!
VarStart	=	((TEXT_MEM+7)/8)*8
;These variables *have* to be defined:
RootVar	=	0*8+VarStart
RemVar	=	1*8+VarStart
ProdVar	=	2*8+VarStart
ExpoVar	=	3*8+VarStart
TempVar	=	4*8+VarStart
TempVar2	=	5*8+VarStart
TempVar3	=	6*8+VarStart
TempVar4	=	7*8+VarStart
;Just define a couple more variables...
VarA	=	8*8+VarStart
VarB	=	9*8+VarStart
;Place the BCDBuffer and other memory locations after all variables:
BCDBuffer	=	10*8+VarStart
BCDBuffEnd	=	BCDBuffer+20
SaveHL	=	BCDBuffEnd+1
SaveDE	=	SaveHL+2
;Where SetUp64 will extract the basic functions
VarFuncs	=	GRAPH_MEM


.org _asm_exec_ram

 nop
 jp Start
 .dw 0
 .dw Title
Title:
 .db "Math64 Tester",0
Start:
;best if you reset the following IY flags...
 res 1,(iy+13)		;don't affect TEXT_MEM
 res 0,(iy+18)		;turn off busy indicator

 call CLEARLCD
 call SetUp64		;extract compressed basic functions to temp mem

 ld hl,$0000
 ld (CURSOR_ROW),hl
 ld hl,EnterString
 call D_ZT_STR		;disply "A?:"
 push hl
 INPUT(VarA)		;input VarA

 ld hl,$0001
 ld (CURSOR_ROW),hl
 pop hl
 call D_ZT_STR		;display "B?:"
 push hl
 INPUT(VarB)		;input VarB

 ld hl,$0002
 ld (CURSOR_ROW),hl
 ROOT(VarA)			;RootVar = int (sqrt)(VarA)
 pop hl
 call D_ZT_STR		;display "sqrt(A)="
 push hl
 DISP(RootVar)		;display RootVar

 ld hl,$0003
 ld (CURSOR_ROW),hl
 ROOT(VarB)			;RootVar = int (sqrt)(VarB)
 pop hl
 call D_ZT_STR		;display "sqrt(B)="
 push hl
 DISP(RootVar)		;display RootVar

 ld hl,$0004
 ld (CURSOR_ROW),hl
;using the exponent function *can* produce an overflow, remember...there's
; only 64-bits here ;)
 EXPONENT(VarA, 2)	;ExpoVar = VarA ^ 2
 pop hl
 call D_ZT_STR		;display "A^2="
 push hl
 DISP(ExpoVar)		;display ExpoVar

 ld hl,$0005
 ld (CURSOR_ROW),hl
;this will more likely produce an overflow than squaring
 EXPONENT(VarB, 7)	;ExpoVar = VarB ^ 7
 pop hl
 call D_ZT_STR		;display "B^7="
 push hl
 DISP(ExpoVar)		;display ExpoVar

 ld hl,$0006
 ld (CURSOR_ROW),hl
;another chance for an overflow...but not *that* likely, maybe
 MULT(VarA, VarB)	;ProdVar = VarA * VarB
 pop hl
 call D_ZT_STR		;display "A*B="
 push hl
 DISP(ProdVar)		;display ProdVar

 ld hl,$0007
 ld (CURSOR_ROW),hl
 DIV(VarA, VarB)	;VarA = int(VarA / VarB)
 pop hl
 call D_ZT_STR		;display "A/B="
 DISP(VarA)			;display VarA

Pause:
 halt
 call GET_KEY
 or a
 jr z,Pause
 ret


;use all the complex functions
#include "SetUp64.h"	;contains SetUp64 and data
#include "root64.h"		;FindRoot64
#include "expo64.h"		;ExponentVar64
#include "mult64.h"		;MultiplyVar64
#include "mult8.h"		;MultiplyVar8
#include "div64.h"		;DivideVar64
#include "div8.h"		;DivideVar8
#include "misc64.h"		;InputVar and DisplayVar

EnterString:
.db "A?:",0
.db "B?:",0
.db "sqrt(A)=",0
.db "sqrt(B)=",0
.db "A^2=",0
.db "B^7=",0
.db "A*B=",0
.db "A/B=",0

.end
